////////////////////////////////////// c++ ////////////////////////////////////
//
//  Module           : parser.hh
//  Description      :
//  Author           : Dino Ahr
//  Email            : Dino.Ahr@informatik.uni-heidelberg.de
//  Copyright        : University of Heidelberg
//  Created on       : Mon Oct 29 08:53:30 2001
//  Last modified by : ahr
//  Last modified on : Tue Oct 30 11:56:32 2001
//  Update count     : 20
//
///////////////////////////////////////////////////////////////////////////////
//
//  Date	Name		Changes/Extensions
//  ----	----		------------------
//
///////////////////////////////////////////////////////////////////////////////


#ifndef PARSER_HH
#define PARSER_HH

#include "symbolTable.hh"
#include "scanner.hh"

/**
 * This class represents a predictive parser according to the following grammar:
 *
 * expr -> term + term | term - term
 * term -> factor * factor | factor / factor |
 *         factor div factor | factor mod factor
 * factor -> ( expr ) | NUM | ID
 * 
 * Nonterminals are realized by private member functions and recursive calls.
 */
class Parser {

private:

  // -----------------------------------------------------
  // -------- V a r i a b l e s --------------------------
  // -----------------------------------------------------

  /**
   * Holds the scanner.
   */
  SymbolTable *m_symbolTable;

  /**
   * Holds the scanner.
   */
  Scanner *m_scanner;

  /**
   * Contains the next token.
   */
  Token m_lookaheadToken;
  
public:

  // -----------------------------------------------------
  // ------- M e t h o d s   ( p u b l i c ) -------------
  // -----------------------------------------------------

  /**
   * Constructor.
   */
  Parser();

  /**
   * Destructor.
   */
  ~Parser();

  /**
   * Starts the parsing process from stdin.
   */
  void parse();

private:

  // -----------------------------------------------------
  // ------- M e t h o d s   ( p r i v a t e ) -----------
  // -----------------------------------------------------

  /**
   * Parse nonterminal \b expr.
   */
  void expr();
  
  /**
   * Parse nonterminal \b term.
   */
  void term();

  /**
   * Parse nonterminal \b factor.
   */
  void factor();

  /**
   * Matches the given token \a t with the current token. If they are equal the
   * next token is read via the scanner, otherwise the parsing is interrupted
   * with a syntax error.
   *
   * @param t The token to be matched with the current token.
   */
  void match(Token t);

  /**
   * Prints a token to stdout.
   *
   * @param t The token to be printed.
   *
   * @param tokenValue The number resp. index of the symbol table for token NUM
   * and ID respectively.
   */
  void emit(Token t, int tokenValue = 0);

}; // class Parser


#endif
